from itertools import zip_longest

from browser import document, window
from .components import Component


class Element:
    _component_config: Component

    def connectedCallback(self):
        self.shadow = self.attachShadow({'mode': 'open'})

        try:
            self.shadow.adoptedStyleSheets = document.adoptedStyleSheets
            adopted = True
        except AttributeError:
            adopted = False

        self.render()

    def render(self):
        self.shadow.innerHTML = self._component_config.template.innerHTML



import browser
from browser import document
from django_brython.testing import location, reverse

from kirui.vdom.render import render


class RenderTest:
    @location(reverse('backend:index'))
    def test_render_base(self):
        el = document.createElement('DIV')
        el.innerHTML = '<span>text</span>'

        el2 = render(el)
        assert isinstance(el2, browser.DOMNode)
        assert el2.outerHTML == '<div><span>text</span></div>'

    @location(reverse('backend:index'))
    def test_render_text_strip(self):
        el = document.createElement('DIV')
        el.innerHTML = """
        <span style="font-weight: bold">
            text
        </span>
        sdfsdfsdf
        """

        el2 = render(el)
        document.body.appendChild(el2)

        print(el.outerHTML)
        print(el2.outerHTML)



from browser import document
from django_brython.testing import location, reverse

from kirui.vdom.components import Component
from kirui.vdom.elements import Element


class ElementTest:
    @location(reverse('backend:index'))
    def test_shadow_inserted(self):
        @Component(
            tag='custom-element',
            template="""<div>
                     content
                     </div>"""
        )
        class CustomElement(Element):
            pass

        obj = CustomElement()
        el = document.createElement('custom-element')
        document.body.appendChild(el)
        assert document.getElementsByTagName('custom-element')[0].shadowRoot.innerHTML == '<div>content</div>', document.getElementsByTagName('custom-element')[0].shadowRoot.innerHTML



import json
from io import StringIO

from samon.elements import BaseElement
from samon.render import RenderedElement


class KrApp(BaseElement):
    def to_xml(self, io: StringIO, indent: int, rendered_element: RenderedElement):

        with rendered_element.frame(io, indent):
            print('<script type="text/javascript">', file=io, end='')
            print(f'var ${self.xml_attrs["id"]}_data = `', file=io, end='')
            rendered_element.to_jsx(io)
            print('`;</script>', file=io)


"""import importlib
import inspect
import sys

from browser import document, window
from kirui.app import App
from kirui.components.layout import SidebarLayout


# Reloading components HMR hook #
def rerender(event):
    module = sys.modules[event.detail['module_name']]

    importlib.reload(module)
    klasses = [m[1] for m in inspect.getmembers(module, inspect.isclass) if m[1].__module__ == event.detail['module_name']]
    for klass in klasses:
        if hasattr(klass, '_component_config'):
            for el in document.getElementsByTagName(klass._component_config.tag):
                el._component_config = klass._component_config
                el.render()


document.bind('reload_brython_module@after', rerender)


def entrypoint():
    app = App(components=[SidebarLayout])
    app.mount('#app')


if __name__ == '__main__':
    entrypoint()
"""

from browser import ajax, webcomponent, window, document


class Example:
    def connectedCallback(self):
        self.attachShadow({'mode': 'open'})
        self.shadowRoot.innerHTML = 'brython webcomponent'


webcomponent.define('kr-example', Example)

from kirui.app import App
from kirui.components.layout import SidebarLayout

def entrypoint():
    """el = document.createElement('DIV')
    el.innerHTML = '<kr-example attr="value">sldf</kr-example>'
    document.getElementById('app').replaceWith(el)"""

    app = App(components=[Example, SidebarLayout])
    app.mount('#app')



from django.test import LiveServerTestCase
from selenium import webdriver

from .brython import ComponentTest


class TestComponent(LiveServerTestCase, ComponentTest):
    def setUp(self):
        self.driver = webdriver.Chrome()

    def tearDown(self):
        self.driver.quit()



from django.contrib import admin
from django.urls import path, include


urlpatterns = [
    path('backend/', include('kirui_devserver.backend.urls', namespace='backend')),
    path('brython/', include('django_brython.urls', namespace='brython')),
]



from django.conf import settings
from django.middleware import csrf
from django.urls import reverse
from django.utils.module_loading import import_string


class DjangoSamonBinding:
    def __init__(self, request):
        self.request = request

    def url(self, view_name, *args, **kwargs):
        return reverse(view_name, args=args, kwargs=kwargs)

    @property
    def csrf_token(self):
        return csrf.get_token(self.request)

    def static(self, path):
        return f'{settings.STATIC_URL}{path}'


def djsamon(request):
    klass = getattr(settings, 'DJANGO_SAMON_BINDING_CLASS', 'django_kirui.context_processors.DjangoSamonBinding')
    klass = import_string(klass)

    return {
        'djsamon': klass(request)
    }



from copy import copy

from .registry import registry
from .loaders import BaseLoader
from .parser import DefaultParser
from .template import Template


class Environment:
    DEFAULT_TEMPLATE_CLASS = Template

    def __init__(self, loader: BaseLoader):
        self.loader = loader
        self.registry = copy(registry)
        self.template_class = self.DEFAULT_TEMPLATE_CLASS

    def get_template(self, template_name):
        src, source_path = self.loader.get_source(template_name)
        parser = DefaultParser(environment=self)
        template = parser.parse(src, template_name=template_name)
        template.source_path = source_path

        return template





def minify(element):
    for child in element.childNodes:
        if child.nodeType == 3:
            text = child.textContent
            text = text.replace('\r\n', ' ')
            text = text.replace('\n', ' ')
            while '  ' in text:  # állítólag ez a leggyorsabb megoldás
                text = text.replace('  ', ' ')
            if text == ' ':
                element.removeChild(child)
                continue

            text = text.lstrip()
            child.textContent = text
        else:
            minify(child)



from watchdog.events import PatternMatchingEventHandler
from .utils import get_md5


class BaseEventHandler(PatternMatchingEventHandler):
    def handle(self, event): # pragma: no cover
        pass

    @property
    def watched_paths(self):
        return []

    def __init__(self):
        super(BaseEventHandler, self).__init__(ignore_directories=True)
        self.history = dict()

    def _remove_from_history(self, path):
        if path in self.history:
            del self.history[path]

    def _update_history(self, path):
        md5hash = get_md5(path)

        if self.history.get(path) == md5hash:
            return False

        self.history[path] = md5hash
        return True

    def on_deleted(self, event):
        self._remove_from_history(event.src_path)
        self.handle(event)

    def on_moved(self, event):
        self._remove_from_history(event.src_path)
        self._update_history(event.dest_path)
        self.handle(event)

    def on_modified(self, event):
        if self._update_history(event.src_path):
            self.handle(event)



import datetime

from django.forms import widgets, fields
from samon.elements import BaseElement, AnonymusElement
from samon.constants import XML_NAMESPACE_DATA_BINDING
from samon.expressions import Bind
from samon.render import RenderedElement
from django_kirui.widgets import CheckboxSwitch


widgets.TextInput.input_type = 'kr-form-input'
widgets.NumberInput.input_type = 'kr-number-input'
widgets.Select.input_type = 'kr-form-select'
widgets.SelectMultiple.input_type = 'kr-multi-select-checkbox'
widgets.CheckboxSelectMultiple.input_type = 'kr-multi-select-checkbox'
widgets.CheckboxInput.input_type = 'kr-checkbox-switch'
widgets.DateInput.input_type = 'kr-date-input'
widgets.Textarea.input_type = 'kr-rich-textbox'


class RenderedField(RenderedElement):
    def _eval_node_attributes(self, context) -> dict:
        retval = super(RenderedField, self)._eval_node_attributes(context)
        self.bf: fields.BoundField = retval.pop('boundField')

        """if self.bf.field.initial:
            retval['value'] = self.bf.field.initial
        else:
            if hasattr(self.bf.field, 'choices') and (choices := list(self.bf.field.choices)):
                if hasattr(choices[0][0], 'value'):
                    retval['value'] = choices[0][0].value
                else:
                    retval['value'] = choices[0][0]"""

        retval['value'] = self.bf.value()

        # TODO: serializálásnál a datetime nem alakítható JSON-ná, ezt az átalakítást máshova kellene tenni
        if 'value' in retval.keys() and isinstance(retval['value'], datetime.datetime):
            retval['value'] = retval['value'].strftime('%Y-%m-%d')

        if self.bf.name in self.bf.form.errors.keys():
            retval['error'] = list(self.bf.form.errors[self.bf.name])[0]

        retval['widget'] = self.bf.field.widget.input_type
        retval['label'] = self.bf.field.label
        retval['required'] = self.bf.field.required
        retval['id'] = f'id_{self.bf.name}'
        retval['name'] = self.bf.name
        retval['onChange'] = f"{retval.pop('formReactContextRef')}.handleInputChange"

        return retval

    @property
    def children(self):
        if choices := getattr(self.bf.field, 'choices', None):
            value = self.node_attributes.get('value', None)
            for choice in choices:
                el = BaseElement(xml_tag='option', xml_attrs={})

                if hasattr(choice[0], 'value'):
                    el.xml_attrs['value'] = choice[0].value
                else:
                    el.xml_attrs['value'] = choice[0]

                if self.node_attributes['name'] == 'topics':
                    print(el.xml_attrs['value'].__class__)

                if isinstance(value, list) and el.xml_attrs['value'] in value:
                    el.xml_attrs['selected'] = True

                el.children = [AnonymusElement(choice[1])]
                yield RenderedElement(el, context=self._context)

        for child in super().children:
            yield child


class KrField(BaseElement):
    RENDERED_ELEMENT_CLASS = RenderedField
    TAG_NAME = 'kr-form-field'


class RenderedForm(RenderedElement):
    @property
    def children(self):
        if self._element.context_var_name:
            form_obj = self._context[self._element.context_var_name]

            for name, field in form_obj.fields.items():
                bf = form_obj[name]
                xml_attrs = {
                    (None, 'name'): name,
                    (None, 'boundField'): bf,
                    (None, 'formReactContextRef'): self._element.xml_attrs['reactContextRef'],
                    (None, 'label-width'): self._element.xml_attrs['label-width'],
                    (None, 'field-width'): self._element.xml_attrs['field-width']
                }
                kr_field = KrField(xml_tag=KrField.TAG_NAME, xml_attrs=xml_attrs)
                yield RenderedField(kr_field, self._context)

        for child in super().children:
            yield child


class KrForm(BaseElement):
    RENDERED_ELEMENT_CLASS = RenderedForm

    def _parse_xml_attrs(self, xml_attrs):
        attrs = super()._parse_xml_attrs(xml_attrs)

        form_obj = attrs.pop(f'{{{XML_NAMESPACE_DATA_BINDING}}}object', None)
        if form_obj is None:
            var_name = None
        else:
            var_name = form_obj.expr

        if 'method' not in attrs.keys():
            attrs['method'] = 'POST'

        if f'{{{XML_NAMESPACE_DATA_BINDING}}}csrfmiddlewaretoken' not in attrs.keys():
            attrs[f'{{{XML_NAMESPACE_DATA_BINDING}}}csrfmiddlewaretoken'] = Bind(expr='djsamon.csrf_token')

        if 'reactContextRef' not in attrs.keys():
            attrs['reactContextRef'] = var_name

        if 'onSubmit' not in attrs.keys():
            attrs['onSubmit'] = f"{attrs['reactContextRef']}.handleSubmit"

        self.context_var_name = var_name
        return attrs
